# Generated by Django 2.2.5 on 2019-09-23 15:14

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('sites_microsoft_auth', '0004_auto_20190923_1512'),
    ]

    operations = [
        migrations.AlterField(
            model_name='microsoftaccount',
            name='site',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='sites.Site'),
        ),
        migrations.AlterField(
            model_name='xboxliveaccount',
            name='site',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='sites.Site'),
        ),
    ]
