# Generated by Django 2.2.5 on 2019-09-23 13:01

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('sites', '__latest__'),
    ]

    operations = [
        migrations.CreateModel(
            name='MicrosoftAccount',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('microsoft_id', models.CharField(max_length=64, verbose_name='microsoft account id')),
            ],
        ),
        migrations.CreateModel(
            name='SiteConfiguration',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('login_enabled', models.BooleanField(default=False)),
                ('login_type', models.CharField(choices=[('ma', 'Microsoft Account'), ('xbl', 'Xbox Live Account')], max_length=2)),
                ('tenant_id', models.CharField(default='common', max_length=80)),
                ('client_id', models.CharField(blank=True, default='', max_length=80)),
                ('client_secret', models.CharField(blank=True, default='', max_length=80)),
                ('extra_scopes', models.CharField(blank=True, default='', max_length=80)),
                ('auto_create', models.BooleanField(default=True)),
                ('register_inactive_admin', models.BooleanField(default=False)),
                ('xbl_sync_username', models.BooleanField(default=False)),
                ('auto_replace_accounts', models.BooleanField(default=False)),
                ('authenticate_hook', models.CharField(blank=True, default='', max_length=100)),
                ('callback_hook', models.CharField(blank=True, default='', max_length=100)),
            ],
        ),
        migrations.CreateModel(
            name='XboxLiveAccount',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('xbox_id', models.CharField(max_length=32, unique=True, verbose_name='xbox user id')),
                ('gamertag', models.CharField(max_length=16, verbose_name='xbox live gamertag')),
            ],
        ),
    ]
