# Generated by Django 2.2.5 on 2019-09-23 15:35

from django.db import migrations


def forward(apps, schema_editor):
    SiteConfiguration = apps.get_model('sites_microsoft_auth', 'SiteConfiguration')
    Site = apps.get_model('sites', 'Site')
    for site in Site.objects.all():
        SiteConfiguration.objects.get_or_create(site=site)


class Migration(migrations.Migration):

    dependencies = [
        ('sites_microsoft_auth', '0005_auto_20190923_1514'),
    ]

    operations = [
        migrations.RunPython(forward, migrations.RunPython.noop)
    ]
